use Test::More tests => 1;
BEGIN { use_ok( 'POE::Component::Client::NSCA' ); };
diag( "Testing POE::Component::Client::NSCA-$POE::Component::Client::NSCA::VERSION, POE-$POE::VERSION, Perl $], $^X" );
